<?php

/**
 * Class OcConnectionStatic
 * Handles the background update queue of json data.
 *
 */
class OcConnectionStatic {

    public static $exec_chain = array();
    public static $key;
    protected static $_instance;

    public static function getInstance() {
        if ( is_null( self::$_instance ) ) {
            $key = "cache_update_queue_";
            $key .= substr( str_shuffle( "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ" ), 0, 10 );

            self::$key = $key;
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    public function __construct() {
    }

    /**
     * On desctruct execute cli command to update needed caces.
     */
    public function __destruct() {

        if ( count( OcConnectionStatic::$exec_chain ) > 0 ) {
            $cmd = PHP_BIN . ' ' . dirname( __FILE__ ) . '/cache_update.php ' . OcConnectionStatic::$key . ' ' . $_SERVER['HTTP_HOST'] . ' > /dev/null &';
            exec( $cmd );
        }
    }
}

/**
 * Class OcConnection
 *
 * Class to enable communication with Open Content
 *
 * Use wrapping OcAPI class to interact with OC
 *
 * @author Infomaker Scandinavie AB
 *
 */
class OcConnection {

    const TRANSIENT_KEY = 'ew_t';
    const TRANSIENT_EXPIRE_TIME = PHP_OB_CACHE_TTL;

    private $remote_args;
    protected $oc_url;
    private $oc_base_url;
    private $return_array = array();
    private $open_content;
    private $exec_chain = array();

    /**
     * Public getter for the OC Base URL
     *
     * @scope public
     *
     * @return null|string
     */
    public function getOcBaseUrl() {
        return isset( $this->oc_base_url ) ? $this->oc_base_url : null;
    }

    /**
     * Public getter for the full OC URL
     *
     * @scope public
     *
     * @return null|string
     */
    public function getOcUrl() {
        return isset( $this->oc_url ) ? $this->oc_url : null;
    }

    /**
     * Public setter for the OC URL
     *
     * @scope public
     *
     * @param $url
     */
    public function setOcUrl( $url ) {
        $this->oc_url = $url;
    }

    /**
     * Public getter for the OC Auth
     *
     * @scope public
     *
     * @return null
     */
    function getOcAuth() {
        return isset( $this->oc_auth ) ? $this->oc_auth : null;
    }

    /**
     * Public getter for cache TTL
     *
     * @scope public
     *
     * @return int
     */
    public function getCacheTTL() {
        return $this->open_content->getTimeToCacheJson();
    }

    /**
     * Publig getter for remote args.
     * @return array
     */
    public function getRemoteArgs() {
        return $this->remote_args;
    }

    /**
     * Public constructor
     *
     * @scope public
     *
     * @param OpenContent $oc
     * @param string $query
     * @param string $start
     * @param string $limit
     * @param string $sort
     */
    function __construct( OpenContent $oc, $query = "", $start = "", $limit = "", $sort = "" ) {
        if ( !empty( $query ) ) {
            trigger_error( 'Oc_Connection is being used in a deprecated way, use OcApi instead', E_STRICT );
        }
        $this->open_content = $oc;
        $this->oc_base_url = $oc->getOcBaseUrl();
        $this->remote_args = array(
            'headers' => array(
                'Authorization' => 'Basic ' . base64_encode( $oc->getOcUserName() . ':' . $oc->getOcPassword() )
            ),
            'timeout' => 20
        );
        $this->oc_url = $this->prep_oc_query( $query, $start, $limit, $sort );

        OcConnectionStatic::getInstance();
    }

    /**
     * Private function called to sanitize and prepare the given query
     *
     * @scope private
     *
     * @param string $query
     * @param string $start
     * @param string $limit
     * @param string $sort
     *
     * @return null|string
     */
    protected function prep_oc_query( $query = "", $start = "", $limit = "", $sort = "" ) {
        $prepared_query = $this->_build_query_string( $query, $start, $limit, $sort, $url_encoded = true );

        if ( $prepared_query !== "" ) {
            return $this->open_content->getOcBaseUrl() . "search?" . $prepared_query;
        } else {
            return $this->open_content->getOcBaseUrl();
        }
    }

    /**
     * Function to prepare a Suggest query
     *
     * @scope public
     *
     * @param        $selector
     * @param        $query
     * @param string $start
     * @param int $limit
     */
    public function prepare_oc_suggest_query( $selector, $query, $start = '', $limit = 25 ) {
        if ( isset( $selector ) ) {
            $this->oc_url = $this->open_content->getOcBaseUrl() . 'suggest?field=' . $selector;
        }

        $prepared_query = $this->_build_query_string( $query, $start, $limit, $sort = '', $url_encode = true );

        if ( $prepared_query !== '' ) {
            $this->oc_url .= '&' . $prepared_query;
        }
    }

    /**
     * Function to build query string
     *
     * @scope    protected
     *
     * @param      $query
     * @param      $start
     * @param      $limit
     * @param      $sort
     * @param bool $url_encoded
     *
     * @return string
     */
    protected function _build_query_string( $query, $start, $limit, $sort, $url_encoded = false ) {

        if ( get_option( 'timezone_string' ) ) {
            date_default_timezone_set( get_option( 'timezone_string' ) );
        }

        $prepared_query = "";

        if ( $query !== "" ) {

            if ( class_exists( 'EnvSettings' ) ) {
                $env_settings = New EnvSettings();
                $env_query = $env_settings->get_env_query();

                if ( $env_query !== '' ) {
                    $query .= ' ' . trim( $env_query );
                }
            }

            $first_char = substr( $query, 0, 1 );

            if ( $first_char === "/" ) {
                $query = substr( $query, 1 );
            }

            $url_start = "";
            if ( isset( $start ) && $start !== "" ) {
                $url_start = "&start=" . $start;
            }

            $url_limit = "";
            if ( isset( $limit ) && $limit !== "" ) {
                $url_limit = "&limit=" . $limit;
            }

            $url_sort = "";
            if ( isset( $sort ) && $sort !== "" ) {
                $url_sort = "&sort.name=" . $sort;
            }

            /*
             * Shortcode Param replace segment
             */
            if ( isset( $_GET["date"] ) ) {
                $date = wp_filter_nohtml_kses( $_GET["date"] );
            } else {
                $date = gmdate( "Y-m-d" );
            }
            $gm_start_date = gmdate( "Y-m-d\TH:i:s\Z", strtotime( $date . '00:00:00' ) );
            $gm_end__date = gmdate( "Y-m-d\TH:i:s\Z", strtotime( $date . '23:59:59' ) );

            if ( isset( $_GET["section"] ) ) {
                $section = '"' . wp_filter_nohtml_kses( $_GET["section"] ) . '"';
            } else {
                $section = '* OR (*:* -ShopSection:*)';
            }

            if ( isset( $_GET["tags"] ) ) {
                $tags = '"' . wp_filter_nohtml_kses( $_GET["tags"] ) . '"';
            } else {
                $tags = '* OR (*:* -Tags:*)';
            }

            if ( isset( $_GET["author"] ) ) {
                $author = '"' . wp_filter_nohtml_kses( $_GET["author"] ) . '"';
            } else {
                $author = '* OR (*:* -Author:*)';
            }

            if ( isset( $_GET["q"] ) ) {
                $text = '' . wp_filter_nohtml_kses( $_GET["q"] ) . '';
            } else if ( isset( $_GET["text"] ) ) {
                $text = '' . wp_filter_nohtml_kses( $_GET["text"] ) . '';
            } else {
                $text = '*';
            }

            $date_from = '';
            $date_to = '';
            if ( isset( $_GET["date_from"], $_GET["date_to"] ) ) {
                $date_from = wp_filter_nohtml_kses( $_GET["date_from"] );
                $date_to = wp_filter_nohtml_kses( $_GET["date_to"] );
            } else {
                $field_pos = strpos( $query, '[param_date_interval]' );
                // TODO: Fix field mapping here so that updated is not hardcoded
                $query = preg_replace( "/\"?\[param_date_interval\]\"?/i", '* OR (*:* -Pubdate:*)', $query );
            }

            global $oc_widget_inject_query;
            $inject_query = '';
            if ( isset( $oc_widget_inject_query ) ) {
                $inject_query = $oc_widget_inject_query;
            }

            global $oc_api_base_query;
            $inject_base_query = '';
            if ( isset( $oc_api_base_query ) ) {
                $inject_base_query = $oc_api_base_query;
            }

            // If a global parameter for start is set, use it instead.
            global $oc_widget_inject_start;
            if ( isset( $oc_widget_inject_start ) ) {
                $url_start = "&start=" . $oc_widget_inject_start;
            }

            //Shortcode replace
            $query = preg_replace( "/\"?\[param_date\]\"?/i", "[" . $gm_start_date . ' TO ' . $gm_end__date . "]", $query );
            $query = preg_replace( "/\"?\[param_date_interval\]\"?/i", "[" . $date_from . "T00:00:00Z TO " . $date_to . "T23:59:59Z]", $query );
            $query = preg_replace( "/\"?\[param_section\]\"?/i", $section, $query );
            $query = preg_replace( "/\"?\[param_tags\]\"?/i", $tags, $query );
            $query = preg_replace( "/\"?\[param_author\]\"?/i", $author, $query );
            $query = preg_replace( "/\"?\[param_text\]\"?/i", $text, $query );
            $query = preg_replace( "/\"?\[param_inject_query\]\"?/i", $inject_query, $query );
            $query = preg_replace( "/\"?\[oc_api_base_query\]\"?/i", $inject_base_query, $query );

            if ( $url_encoded == true ) {
                $prepared_query = "q=" . urlencode( $query ) . $url_start . $url_limit . $url_sort;
            } else {
                $prepared_query = "q=" . $query . $url_start . $url_limit . $url_sort;
            }

        }

        return $prepared_query;
    }

    /**
     * Function to get remote content, using build in WP remote_get functions
     *
     * @scope protected
     *
     * @param $url
     *
     * @return null|string
     */
    protected function _get_remote_content( $url ) {

        $start = microtime( true );


        $api_response = wp_remote_get( $url, $this->remote_args );
        if ( !is_wp_error( $api_response ) ) {
            $http_status = wp_remote_retrieve_response_code( $api_response );

            if ( $http_status == "200" ) {
                $json_content = wp_remote_retrieve_body( $api_response );

                $debug = $this->get_debug_data_for_analyzer( $start );
                do_action( 'analyzer_add_data', [
                    'name' => 'Call from Oc_Connection',
                    'debug' => $debug
                ], 'OC' );

                return $json_content;
            }
            if ( $http_status == '401' ) {

                $debug = $this->get_debug_data_for_analyzer( $start );
                do_action( 'analyzer_add_data', [
                    'name' => 'Call from Oc_Connection with 401 response code',
                    'debug' => $debug
                ], 'OC_ERROR' );
                return '401';
            } else {
                return null;
            }
        }

        return null;
    }

    protected function get_debug_data_for_analyzer( $start = null, $backtrace = 20 ) {
        if ( null === $start ) {
            $start = microtime( true );
        }

        $data = debug_backtrace( DEBUG_BACKTRACE_PROVIDE_OBJECT, $backtrace );
        $data[1]['time'] = [ 0 => microtime( true ) - $start ];
        return $data;

    }

    /**
     * function to get remote data, using build in WP remote_get functions
     *
     * @scope protected
     *
     * @param $url
     *
     * @return null|string
     */
    protected function _get_remote_data( $url ) {
        $start = microtime( true );

        $api_response = wp_remote_get( $url, $this->remote_args );

        if ( !is_wp_error( $api_response ) ) {
            $http_status = wp_remote_retrieve_response_code( $api_response );

            if ( $http_status == "200" ) {
                $debug = $this->get_debug_data_for_analyzer( $start );
                do_action( 'analyzer_add_data', [
                    'name' => 'Call from Oc_Connection',
                    'debug' => $debug
                ], 'OC' );
                return $api_response;
            }
            if ( $http_status == '401' ) {
                $debug = $this->get_debug_data_for_analyzer( $start );
                do_action( 'analyzer_add_data', [
                    'name' => 'Call from Oc_Connection 401 data',
                    'debug' => $debug
                ], 'OC_ERROR' );
                return '401';
            } else {
                return null;
            }
        }

        return null;
    }

    /**
     * Function to get json results from cache, if exists, else from Open Content and then store in cache
     *
     * @scope protected
     *
     * @param $url
     *
     * @return bool|null|string
     */
    protected function get_json_result( $url = null ) {
        if ( $url === null ) {
            $url = $this->oc_url;
        }

        $key = md5( $url );
        $result = false;
        $tmp_result = false;

        // Try to get content from cached transient.
        $result = get_transient( OcConnection::TRANSIENT_KEY . '_' . $key );
        $tmp_result = json_decode( $result );

        if ( $tmp_result === null || $tmp_result === false ) { // If no content is found in transient, get it directly from Open Content.
            $result = $this->update_transient_cache( $url, $key );
            $tmp_result = json_decode( $result );
        } else if ( false === get_transient( OcConnection::TRANSIENT_KEY . '_' . $key . '_expire' ) ) { // If timeout for query has occured, fill it up with async script for next user.
            #$scrip_url = get_home_url() . "/wp-content/plugins/Every/includes/cache_update.php";
            #$cmd = 'php ' . dirname( __FILE__ ) . '/cache_update.php ' . urlencode( $scrip_url ) . ' ' .  urlencode( $url ) . ' ' . urlencode( $key );
            #OcConnectionStatic::$exec_chain[] = $cmd . ' > /dev/null';

            #$queue = get_transient(OcConnectionStatic::$key);
            #if(false === $queue) {
            #    $queue = array();
            #}

            OcConnectionStatic::$exec_chain[] = array(
                $url,
                $key
            );
            set_transient( OcConnectionStatic::$key, OcConnectionStatic::$exec_chain, self::TRANSIENT_EXPIRE_TIME );

            #OcConnectionStatic::$exec_chain[] = array($url, $key);

            set_transient( OcConnection::TRANSIENT_KEY . '_' . $key . '_expire', 'true', $this->open_content->getTimeToCacheJson() );
        }

        if ( isset( $tmp_result->uuid ) ) {
            OcCacheHelper::getInstance()->add_to_cache_map( $tmp_result->uuid, $key, self::TRANSIENT_EXPIRE_TIME );
        } else if ( isset( $tmp_result->contentType ) && strtolower( $tmp_result->contentType ) === 'article' ) {
            $uuid = "";
            foreach ( $tmp_result->properties as $prop ) {
                if ( $prop->name === 'uuid' ) {
                    $uuid = $prop->values[0];
                    break;
                }
            }

            OcCacheHelper::getInstance()->add_to_cache_map( $uuid, $key, self::TRANSIENT_EXPIRE_TIME );
        } else if ( isset( $tmp_result->hits ) ) { //Multiple articles
            foreach ( $tmp_result->hits->hits as $hit ) {
                OcCacheHelper::getInstance()->add_to_cache_map( $hit->id, $key, self::TRANSIENT_EXPIRE_TIME );
            }
        }

        return $result;
    }

    /**
     * Function to update a transients cache with content from Open Content.
     * @return [type] [description]
     */
    public function update_transient_cache( $url, $key ) {

        $result = $this->_get_remote_content( $url );
        $this->set_transient_cache( $result, $key );
        //		if ( $result !== null && $result !== '401' ) {
        //			set_transient( OcConnection::TRANSIENT_KEY . '_' . $key, $result, 2 * WEEK_IN_SECONDS );
        //			set_transient( OcConnection::TRANSIENT_KEY . '_' . $key . '_expire', 'true', $this->open_content->getTimeToCacheJson() );
        //
        //		} else {
        //			//trigger_error( 'Error in get_json_result - _get_remote_content, Result: ' . $result, E_USER_NOTICE );
        //            set_transient( OcConnection::TRANSIENT_KEY . '_' . $key . '_expire', 'true', 120 );
        //		}

        return $result;
    }

    /**
     * Helper function to set transient
     * @param $to_cache
     * @param $key
     */
    private function set_transient_cache( $to_cache, $key ) {
        if ( $to_cache !== null && $to_cache !== '401' ) {
            set_transient( OcConnection::TRANSIENT_KEY . '_' . $key, $to_cache, self::TRANSIENT_EXPIRE_TIME );
            set_transient( OcConnection::TRANSIENT_KEY . '_' . $key . '_expire', 'true', $this->open_content->getTimeToCacheJson() );
        } else {
            //trigger_error( 'Error in get_json_result - _get_remote_content, Result: ' . $result, E_USER_NOTICE );
            set_transient( OcConnection::TRANSIENT_KEY . '_' . $key . '_expire', 'true', 120 );
        }
    }

    public function update_transient_notifier_push( $response ) {
        //check all keys that the uuid resides
        $transient_keys = OcCacheHelper::getInstance()->get_transient_keys_by_uuid( $response['uuid'] );
        $response_json = json_decode( $response['response'] );

        if ( sizeof( $transient_keys ) > 0 ) {
            foreach ( $transient_keys as $transient_key => $expire_time ) {
                $json = json_decode( get_transient( OcConnection::TRANSIENT_KEY . '_' . $transient_key ), true );

                if ( isset( $json['hits'] ) ) {

                    foreach ( $json['hits']['hits'] as $hit_key => $hit ) {

                        if ( isset( $hit['id'] ) || isset( $hit['uuid'] ) ) {
                            $uuid = isset( $hit['id'] ) ? $hit['id'] : $hit['uuid'];

                            if ( $uuid === $response['uuid'] ) {
                                $json['hits']['hits'][$hit_key]['versions'][0]['properties'] = $this->format_multi_properties( $response_json->properties );
                            }
                        }
                    }

                    $this->set_transient_cache( json_encode( $json, JSON_UNESCAPED_UNICODE ), $transient_key );
                    OcCacheHelper::getInstance()->update_cache_map( $response['uuid'], $transient_key, self::TRANSIENT_EXPIRE_TIME );
                } else if ( isset( $json['contentType'] ) && strtolower( $json['contentType'] ) === 'article' ) {
                    $json['properties'] = $this->format_single_properties( $response_json->properties );
                    $this->set_transient_cache( json_encode( $json, JSON_UNESCAPED_UNICODE ), $transient_key );
                    OcCacheHelper::getInstance()->update_cache_map( $response['uuid'], $transient_key, self::TRANSIENT_EXPIRE_TIME );
                }

            }
        }
    }

    public function add_transient_notifier_push( $response ) {
        $key = md5( $this->oc_base_url . "objects/{$response['uuid']}/properties" );

        $this->set_transient_cache( $response['response'], $key );
        OcCacheHelper::getInstance()->add_to_cache_map( $response['uuid'], $key, self::TRANSIENT_EXPIRE_TIME );
    }

    public function delete_transient_notifier_push( $uuid ) {
        $ret = false;
        $transient_keys = OcCacheHelper::getInstance()->get_transient_keys_by_uuid( $uuid );

        if ( sizeof( $transient_keys ) > 0 ) {
            $ret = true;
            foreach ( $transient_keys as $transient_key => $expire_time ) {
                $json = json_decode( get_transient( OcConnection::TRANSIENT_KEY . '_' . $transient_key ), true );

                if ( isset( $json['hits'] ) ) {
                    foreach ( $json['hits']['hits'] as $hit_key => $hit ) {

                        if ( $hit['id'] === $uuid ) {
                            unset( $json['hits']['hits'][$hit_key] );
                            OcCacheHelper::getInstance()->remove_from_cache_map( $uuid, $transient_key );
                        }
                    }

                    $this->set_transient_cache( json_encode( $json, JSON_UNESCAPED_UNICODE ), $transient_key );
                } else if ( isset( $json['contentType'] ) && strtolower( $json['contentType'] ) === 'article' ) {

                    delete_transient( OcConnection::TRANSIENT_KEY . '_' . $transient_key );
                    delete_transient( OcConnection::TRANSIENT_KEY . '_' . $transient_key . '_expire' );
                    OcCacheHelper::getInstance()->remove_from_cache_map( $uuid, $transient_key );
                }
            }
        }

        return $ret;
    }

    private function format_single_properties( $properties ) {
        $ret = array();

        foreach ( $properties as $prop ) {
            array_push( $ret, array(
                'name' => $prop->name,
                'values' => $prop->values
            ) );
        }

        return $ret;
    }

    private function format_multi_properties( $properties ) {
        $ret = array();

        foreach ( $properties as $prop ) {
            $key = $prop->name;
            $val = $prop->values;
            $ret[$key] = $val;
        }

        return $ret;
    }


    /**
     * Function to clear cache on Widget
     *
     * @scope public
     *
     * @param $md5_key
     */
    public static function clear_widget_cache( $md5_key ) {
        if ( isset( $md5_key ) ) {
            $result = delete_transient( OcConnection::TRANSIENT_KEY . '_' . $md5_key );
            do_action( 'clear_widget_cache_hook', $result, $md5_key );
        }
    }

    /**
     * Function to update Widget cache
     * This function is called when OC Notifier tells us there has
     * been a CRUD event on an Widget query result
     * OR when a widget is saved
     * This function will flush existing cache and get new JSON from OC and store it in transient
     *
     * @scope public
     *
     * @param bool $oc_query
     */
    public function update_widget_cache( $oc_query = false ) {
        if ( $oc_query === false ) {
            //Widget updated, take action...
            $url = $this->oc_url;

        } else {
            //Notifier calling, take action...
            $url = $this->oc_base_url . 'search?' . $oc_query;
        }

        $key = md5( $url );
        $this->update_transient_cache( $url, $key );
        /*
        $result = $this->_get_remote_content( $url );

        if ( $result !== null && $result !== '401' ) {
            OcConnection::clear_widget_cache( $key );
            $set_trans_result = set_transient( OcConnection::TRANSIENT_KEY . '_' . $key, $result, $this->open_content->getTimeToCacheJson() );
        }

        do_action( 'update_widget_cache_hook', $set_trans_result, $key );
        */
    }

    /**
     * DEPRECATED
     *
     * Function to be called by OC to invalidate cache on ALL Widgets
     * Will trigger widgets to empty their cache
     */
    public function oc_event_clear_cache() {
        require_once( '../../../wp-load.php' );

        $widgets = array();
        $oc_article_widgets = array();
        $oc_article_widgets_custom = array();

        //Get all page widgets (widgets from static pages like Home etc)
        $pages = get_pages();
        foreach ( $pages as $key => $page ) {
            if ( $key !== "wp_inactive_widgets" ) {
                $widget = get_post_meta( $page->ID, '_sidebars_widgets', true );
                array_push( $widgets, $widget );
            }
        }

        //Get all oc article widgets (General widgets set on Articles-widgets page)
        global $sidebars_widgets;
        if ( !empty( $sidebars_widgets ) ) {
            foreach ( $sidebars_widgets as $key => $widget ) {
                if ( $key !== "wp_inactive_widgets" ) {
                    array_push( $oc_article_widgets, $widget );
                }
            }
        }

        //Get all custom oc article widgets (customized widgets on specific articles)
        $args = array( 'post_type' => 'article' );
        $oc_articles = get_posts( $args );
        foreach ( $oc_articles as $key => $article ) {
            if ( $key !== "wp_inactive_widgets" ) {
                array_push( $oc_article_widgets_custom, get_post_meta( $article->ID, '_sidebars_widgets', true ) );
            }
        }

        array_push( $widgets, $oc_article_widgets );
        $widgets = array_merge( $widgets, $oc_article_widgets_custom );

        foreach ( $widgets as $key => $widget_array ) {
            if ( $widget_array === "" ) {
                unset( $widgets[$key] );
            }

            if ( is_array( $widget_array ) && count( $widget_array ) > 0 ) {
                foreach ( $widget_array as $widget_area ) {
                    if ( is_array( $widget_area ) ) {
                        foreach ( $widget_area as $i => $widget_id ) {
                            OcConnection::clear_widget_cache( $widget_id ); //This line will clear the cache
                        }
                    }
                }
            }
        }
    }

    /**
     * Function to get an array with articles from Open Content
     *
     * @scope public
     *
     * @return array
     */
    function get_oc_articles( $create_cpt = true ) {
        $result = $this->get_json_result();
        $this->return_array = array();

        if ( $result != null && $result !== '401' ) {
            $json_result = json_decode( $result );
            $this->latestQueryTotalHits = ( isset( $json_result->hits ) && isset( $json_result->hits->totalHits ) ) ? $json_result->hits->totalHits : 0;

            try {
                foreach ( $json_result->hits->hits as $article ) {
                    array_push( $this->return_array, $this->create_article_object( $article, $create_cpt ) );
                }
            } catch (Exception $e) {
                trigger_error( 'Error in get_oc_articles - Foreach json->hits->hits, Exception: ' . $e->getTraceAsString(), E_USER_ERROR );
            }
        }

        return $this->return_array;
    }

    /**
     * Function to get an array with ads from Open Content
     *
     * @scope public
     *
     * @return array
     */
    function get_oc_ads() {
        $result = $this->get_json_result();
        $this->return_array = array();

        if ( $result != null && $result !== '401' ) {
            $json_result = json_decode( $result );
            try {
                foreach ( $json_result->hits->hits as $ad ) {
                    array_push( $this->return_array, $this->create_ad_object( $ad ) );
                }
            } catch (Exception $e) {
                trigger_error( 'Error in get_oc_ads - Foreach json->hits->hits, Exception: ' . $e->getTraceAsString(), E_USER_ERROR );
            }

        }

        return $this->return_array;
    }

    /**
     * Function to get an array with Images from Open Content
     *
     * @scope public
     *
     * @return array
     */
    function get_oc_images() {
        $result = $this->get_json_result();
        $this->return_array = array();

        if ( $result != null && $result !== '401' ) {
            $json_result = json_decode( $result );
            try {
                foreach ( $json_result->hits->hits as $image ) {
                    array_push( $this->return_array, $this->create_image_object( $image ) );
                }
            } catch (Exception $e) {
                trigger_error( 'Error in get_oc_ads - Foreach json->hits->hits, Exception: ' . $e->getTraceAsString(), E_USER_ERROR );
            }

        }

        return $this->return_array;
    }

    /**
     * function to get full article with related images
     *
     * @scope public
     *
     * @param $article_id
     * @param $prop_arr
     *
     * @return array with an OcArticle and an Array with imagess
     */
    function get_single_article( $article_id, $prop_arr = null ) {

        global $use_oc_cache;

        $query = $this->oc_base_url . "objects/$article_id/properties";

        // If property array is set, add it's values to the query.
        if ( isset( $prop_arr ) ) {
            $c = 0;
            foreach ( $prop_arr as $prop ) {

                if ( $c === 0 ) {
                    $query .= '?name=' . $prop;
                } else {
                    $query .= '&name=' . $prop;
                }

                $c++;
            }
        }

        $this->oc_url = $query;
        if ( $use_oc_cache ) {
            $result = $this->get_json_result();
        } else {
            $result = $this->_get_remote_content( $this->oc_url );
        }

        $this->return_array = array();

        if ( $result != null && $result !== '401' ) {
            $json_result = json_decode( $result );

            $article = new OcArticle();

            foreach ( $json_result->properties as $prop ) {
                $name = strtolower( $prop->name );
                $val = $prop->values;
                $article->$name = $val;
            }

            $this->add_mapped_properties( $article );

            $article_image_uuids = [ ];
            if ( isset( $article->imageuuid ) && is_array( $article->imageuuid ) && count( $article->imageuuid ) > 0 ) {
                $article_image_uuids = $article->imageuuid;
            } else {
                $article_image_uuids = $this->_get_single_article_images( $article->uuid[0] );
            }

            $this->return_array['article'] = $article;
            $this->return_array['article_images'] = $article_image_uuids;

            //Create a new Custom post type from the article object
            $custom_post_type_factory = new OcArticleCustomPostFactory( $article );
        }
        return $this->return_array;
    }

    /**
     * Function to set properties from OC property map
     *
     * @param $article - oc_article object
     */
    function add_mapped_properties( $article ) {
        $prop_map = OcUtilities::get_property_map();

        foreach ( $prop_map as $key => $value ) {
            $key = strtolower( $key );
            $temp = $article->$value;
            $article->$key = $temp;
        }
    }

    /**
     * Function to get all image metadata
     *
     * @scope public
     *
     * @param $image_id
     * @param $prop_arr
     *
     * @return OcImage
     */
    function get_single_image_metadata( $image_id, $prop_arr = null ) {

        $query = $this->oc_base_url . "objects/$image_id/properties";
        // If property array is set, add it's values to the query.
        if ( isset( $prop_arr ) ) {
            $c = 0;
            foreach ( $prop_arr as $prop ) {

                if ( $c === 0 ) {
                    $query .= '?name=' . $prop;
                } else {
                    $query .= '&name=' . $prop;
                }

                $c++;
            }
        }

        $result = $this->get_json_result( $query );
        $image = new OcImage();

        if ( $result != null && $result !== '401' ) {
            $json_result = json_decode( $result );

            foreach ( $json_result->properties as $prop ) {
                $name = strtolower( $prop->name );
                $val = $prop->values;
                $image->$name = $val;
            }
        }
        return $image;
    }

    /**
     * Function to be called by get_single_article to get related images
     *
     *
     * @param $article_id
     *
     * @return array
     */
    private function _get_single_article_images( $article_id ) {
        $query = $this->oc_base_url . "objects/$article_id/relations";

        $result = $this->get_json_result( $query );
        $uuid_arr = array();

        if ( $result != null && $result !== '401' ) {

            $json_result = json_decode( $result );
            if ( count( $json_result->relations ) > 0 ) {

                foreach ( $json_result->relations as $key => $value ) {

                    if ( $value->targetContentType === 'Image' && isset( $value->uuids ) && count( $value->uuids ) > 0 ) {

                        foreach ( $value->uuids as $uuid ) {
                            array_push( $uuid_arr, $uuid );
                        }
                    }
                }
            }
        }

        return $uuid_arr;
    }

    /**
     * Function called by partial view to render image-src
     *
     * @scope public
     *
     * @param int $image_uuid
     * @param string $image_size
     * @param bool $source_file
     *
     * @return null|string
     */
    function get_image( $image_uuid = 0, $image_size = "thumbnail", $source_file = false ) {
        if ( $image_uuid === 0 ) {
            return "fail";
        }

        if ( !$source_file ) {

            $query = $this->oc_base_url . "objects/$image_uuid/files/$image_size";
        } else {

            $query = $this->oc_base_url . "objects/$image_uuid";
        }

        return $this->_get_remote_content( $query );
    }

    /**
     * Function called by article view to display image-ul
     *
     * @scope public
     *
     * @param int $image_uuid
     * @param string $image_size
     * @param bool $source_file
     *
     * @return string
     */
    function get_image_url( $image_uuid = 0, $image_size = "thumbnail", $source_file = false ) {
        if ( $image_uuid === 0 ) {
            return "fail";
        }

        if ( !$source_file ) {

            $query = $this->oc_base_url . "objects/$image_uuid/files/$image_size";
        } else {

            $query = $this->oc_base_url . "objects/$image_uuid";
        }

        return $query;
    }

    private function build_search_query( array $params ) {
        $params = array_replace_recursive( [
            'q' => '',
            'start' => 0,
            'limit' => 15,
            'contenttypes' => [ ],
            'properties' => [ ],
            'sort.name' => null,
            'sort.indexfield' => null,
            'facet' => null,
            'facet.limit' => null,
            'facet.mincount' => null,
            'facet.date.mincount' => null,
        ], $params );

        // We create our own parameters from these arrays
        $properties = $params['properties'];
        $contenttypes = $params['contenttypes'];
        unset( $params['properties'] );
        unset( $params['contenttypes'] );

        $env_settings = new EnvSettings();
        if ( $env_settings->get_use_trashed() ) {
            $params['q'] .= ' AND NOT EveryTrashed:"' . OcUtilities::get_site_shortname() . '"';
        }

        // Remove q-parameter if no query has been made
        $params['q'] = $params['q'] !== '' ? $params['q'] : null;

        // Convert facet-parameter if set
        if ( isset( $params['facet'] ) ) {
            $params['facet'] = $params['facet'] ? 'true' : 'false';
        }

        $query_string = http_build_query( $params );

        // Add contenttypes if specified
        foreach ( $contenttypes as $contenttype ) {
            $query_string .= "&contenttype={$contenttype}";
        }

        // Add property-parameters if properties has been specified.
        if ( count( $properties ) > 0 ) {

            // Add contenttype to the properties as it is needed by the function.
            if ( !in_array( 'contenttype', $properties ) ) {
                array_push( $properties, 'contenttype' );
            }

            // Add the rest of the properties
            foreach ( $properties as $property ) {
                $query_string .= "&property={$property}";
            }
        }

        return "{$this->oc_base_url}search?{$query_string}";
    }

    /**
     * Function to get freetext search from OC
     *
     * @scope public
     *
     * @param        $search_text
     * @param        $include_array
     * @param string $sort
     * @param null $facet_limit
     * @param null $facet_mincount
     * @param bool $create_article
     * @param array $prop_arr
     * @param bool $use_cache
     *
     * @return array
     */
    function text_search( $search_text, $include_array, $sort = '', $facet_limit = null, $facet_mincount = null, $start = '', $limit = '', $create_article = true, $prop_arr = null, $use_cache = false, $use_facet = null ) {

        // Redirect to new search function
        return $this->search( [
            'q' => $search_text,
            'contenttypes' => $include_array,
            'sort.name' => $sort,
            'facet' => $use_facet,
            'facet.limit' => $facet_limit,
            'facet.mincount' => $facet_mincount,
            'facet.date.mincount' => $facet_mincount,
            'start' => $start,
            'limit' => $limit,
            'properties' => $prop_arr
        ], $use_cache, $create_article );
    }

    /**
     * New function for using search against Open Content
     *
     * @param array $options
     * @param bool $use_cache
     * @param bool $create_article
     *
     * @since 0.1
     * @return array|string
     */
    public function search( $params = [ ], $use_cache = true, $create_article = true ) {
        $this->return_array = array();

        $query = $this->_encode_query( $this->build_search_query( $params ) );
        return $this->get_search_result( $query, $use_cache, $create_article );
    }

    private function get_search_result( $query, $use_cache, $create_article ) {
        $result = null;

        if ( $use_cache ) {
            $result = $this->get_json_result( $query );
        } else {
            $result = $this->_get_remote_content( $query );
        }

        if ( $result != null && $result !== '401' ) {
            $json_result = json_decode( $result );
            $this->return_array['duration'] = $json_result->stats->duration;
            $this->return_array['hits'] = $json_result->hits->totalHits;
            $this->return_array['facet'] = $json_result->facet;

            foreach ( $json_result->hits->hits as $object ) {
                switch (strtolower( $object->versions[0]->properties->contenttype[0] )) {
                    case 'ad':
                        array_push( $this->return_array, $this->create_ad_object( $object ) );
                        break;
                    case 'article':
                        array_push( $this->return_array, $this->create_article_object( $object, $create_article ) );
                        break;
                    case 'image':
                        array_push( $this->return_array, $this->create_image_object( $object ) );
                        break;
                    default:
                        array_push( $this->return_array, $this->create_oc_object( $object ) );
                        break;
                }
            }
        }

        return $this->return_array;
    }

    /**
     * Function to get Sections from Open Content
     *
     * @return array
     */
    function get_oc_suggest() {
        $result = $this->get_json_result();
        $return_array = array();

        if ( $result != null && $result !== '401' ) {
            $json_result = json_decode( $result );
            try {
                foreach ( $json_result->facetFields[0]->terms as $section ) {
                    array_push( $return_array, $section );
                }
            } catch (Exception $e) {
                trigger_error( 'Error in get_oc_sections - Foreach json->hits->hits, Exception: ' . $e->getTraceAsString(), E_USER_ERROR );
            }

        }

        return $return_array;
    }

    /**
     * Function to create Article object
     *
     * @scope protected
     *
     * @param $article
     *
     * @return OcArticle
     */
    protected function create_article_object( $article, $create_cpt = true ) {
        $oc_article = new OcArticle();
        foreach ( $article->versions[0]->properties as $prop => $val ) {
            $name = strtolower( $prop );
            $oc_article->$name = $val;
        }
        if ( !isset( $oc_article->uuid ) || !isset( $oc_article->uuid[0] ) ) {
            $oc_article->uuid = array( $article->id );
        }

        $this->add_mapped_properties( $oc_article );

        //Create an instance of our custom post type if wanted
        if ( $create_cpt ) {
            new OcArticleCustomPostFactory( $oc_article );
        }

        return $oc_article;
    }


    /**
     * function to create Ad object
     *
     * @scope protected
     *
     * @param $ad
     *
     * @return OcAd
     */
    protected function create_ad_object( $ad ) {
        $oc_ad = new OcAd();

        foreach ( $ad->versions[0]->properties as $prop => $val ) {
            $name = strtolower( $prop );
            $oc_ad->$name = $val;
        }
        if ( !isset( $oc_ad->uuid ) ) {
            $oc_ad->uuid = array( $ad->id );
        }

        return $oc_ad;
    }

    /**
     * function to create Image object
     *
     * @scope protected
     *
     * @param $image
     *
     * @return OcImage
     */
    protected function create_image_object( $image ) {
        $oc_img = new OcImage();

        foreach ( $image->versions[0]->properties as $prop => $val ) {
            $name = strtolower( $prop );
            $oc_img->$name = $val;
        }

        if ( !isset( $oc_img->uuid ) ) {
            $oc_img->uuid = array( $image->id );
        }

        $this->add_mapped_properties( $oc_img );

		return $oc_img;
	}

    /**
     * function to create Page object
     *
     * @scope protected
     *
     * @param $page
     *
     * @return OcPage
     */
    protected function create_page_object( $page ) {
        $oc_page = new OcPage();

        foreach ( $page->versions[0]->properties as $prop => $val ) {
            $name = strtolower( $prop );
            $oc_page->$name = $val;
        }

        if ( !isset( $oc_page->uuid ) ) {
            $oc_page->uuid = array( $page->id );
        }

        return $oc_page;
    }

    /**
     * function to create Oc object
     *
     * @scope protected
     *
     * @param $object
     *
     * @return OcObject
     */
    protected function create_oc_object( $object ) {
        $oc_object = new OcObject();

        foreach ( $object->versions[0]->properties as $prop => $val ) {
            $name = strtolower( $prop );
            $oc_object->$name = $val;
        }

        if ( !isset( $oc_object->uuid ) ) {
            $oc_object->uuid = array( $object->id );
        }

        return $oc_object;
    }

    /**
     * Function to get OC Sort options
     *
     * @scope public
     *
     * @return array|mixed
     */
    function get_oc_sort_options() {

        $key = 'ew_sort_options';
        if( false === ( $this->return_array = get_transient( $key ) ) ) {

            $query = $this->oc_base_url . "sortings";
            $result = $this->_get_remote_content( $query );

            if ( $result !== null && $result !== '401' ) {
                $this->return_array = json_decode( $result );
            } else {
                $this->return_array = array();
            }

            set_transient( $key, $this->return_array, 300 );
        }

        return $this->return_array;
    }

    /**
     * Function to test oc query with Ajax call
     *
     * @scope public
     *
     * @return array
     */
    public function ajax_test_query() {
        $result = $this->_get_remote_content( $this->oc_url );
        $this->return_array = array();

        if ( $result != null && $result !== '401' ) {

            $json_result = json_decode( $result );
            $hits = $json_result->hits;

            //If we get a response from OC but no hits
            if ( $hits->totalHits == 0 ) {
                $this->return_array['No Matches'] = __( "Given query returns no matching objects from Open Content" );
            } else {
                foreach ( $hits->hits as $object ) {
                    switch (strtolower( $object->versions[0]->properties->contenttype[0] )) {
                        case 'ad':
                            !isset( $this->return_array['Ads'] ) ? $this->return_array['Ads'] = 0 : null;
                            $this->return_array['Ads']++;
                            break;
                        case 'article':
                            !isset( $this->return_array['Articles'] ) ? $this->return_array['Articles'] = 0 : null;
                            $this->return_array['Articles']++;
                            break;
                        case 'image':
                            !isset( $this->return_array['Images'] ) ? $this->return_array['Images'] = 0 : null;
                            $this->return_array['Images']++;
                            break;
                        case 'page':
                            !isset( $this->return_array['Pages'] ) ? $this->return_array['Pages'] = 0 : null;
                            $this->return_array['Pages']++;
                            break;
                        case 'video':
                            !isset( $this->return_array['Video'] ) ? $this->return_array['Video'] = 0 : null;
                            $this->return_array['Video']++;
                            break;
                        default:
                            !isset( $this->return_array['Others'] ) ? $this->return_array['Others'] = 0 : null;
                            $this->return_array['Others']++;
                            break;
                    }
                }
            }

        } elseif ( $result === '401' ) //Not authorized
        {
            $this->return_array['Authentication'] = __( 'Invalid Open Content credentials. Check Open Content Settings' );
        } else //No respons from OC or Null object is returned
        {
            $this->return_array['No Matches'] = __( "No Response" );
        }

        return $this->return_array;
    }

    /**
     * Function to get Content types with Ajax
     *
     * @scope public
     *
     * @return array
     */
    public function ajax_get_content_types() {
        $query = $this->oc_base_url . 'contenttypes';
        $result = $this->_get_remote_content( $query );
        $this->return_array = array();

        if ( $result != null && $result != '401' ) {
            $json_result = json_decode( $result );

            foreach ( $json_result->contentTypes as $content_type ) {
                $this->return_array[$content_type->name] = array();

                foreach ( $content_type->properties as $property ) {
                    array_push( $this->return_array[$content_type->name], $property->indexFields );
                }
            }
        } else if ( $result === '401' ) {
            $this->return_array['Authentication'] = __( 'Invalid Open Content credentials. Check "Settings - Open Content"' );
        } else {
            $this->return_array['No Matches'] = __( "No Content types found in Open Content" );
        }

        return $this->return_array;
    }

    /**
     * Function to get Index fields from Open Content
     *
     * @return array
     */
    public function get_index_fields() {
        $query = $this->oc_base_url . 'admin/indexfields';
        $result = $this->_get_remote_content( $query );
        $this->return_array = array();

        if ( $result !== null && $result !== '401' ) {
            $json_result = json_decode( $result );

            foreach ( $json_result->indexFields as $index_field ) {

                if ( !empty( $property->indexFields ) ) {
                    array_push( $this->return_array, $index_field->name );
                }
            }
        } else if ( $result === '401' ) {
            $this->return_array['Authentication'] = __( 'Could not fetch Index fields due to Invalid Open Content credentials. Check "Settings - Open Content"' );
        } else {
            $this->return_array['No Matches'] = __( "No Index Fields found in Open Content" );
        }

        return $this->return_array;
    }

    /**
     * Function to get Suggest results with AJAX
     *
     * @scope public
     *
     * @param $field
     * @param $q
     * @param $incompleteWord
     * @param $incompleteWordInText
     *
     * @return null|string
     */
    public function ajax_get_suggest( $field, $q, $incompleteWord, $incompleteWordInText ) {

        $query_str = 'suggest/';
        if ( $field !== null ) {
            $query_str = add_query_arg( 'field', urlencode( $field ), $query_str );
        }

        // TODO: This one is causing all kind of problems. Check if it is needed.
        /*if ( $q !== null && $field !== null ) {
            $query_str = add_query_arg( 'q', urlencode( $field . ':' . $q ), $query_str );
        } else*/
        if ( $q !== null ) {
            $query_str = add_query_arg( 'q', urlencode( $q ), $query_str );
        }

        if ( $incompleteWord !== null ) {
            $query_str = add_query_arg( 'incompleteWord', urlencode( $incompleteWord ), $query_str );
        }

        if ( $incompleteWordInText !== null ) {
            $query_str = add_query_arg( 'incompleteWordInText', urlencode( $incompleteWordInText ), $query_str );
        }

        $this->oc_url = $this->open_content->getOcBaseUrl() . $query_str;

        $result = $this->get_json_result( $this->oc_url );
        if ( $result !== null && $result !== '401' ) {
            return $result;
        }

        die();
    }

    /**
     * Function to url encode parameter values in a URL string
     *
     * @scope private
     *
     * @param $query
     *
     * @return string
     */
    private function _encode_query( $query ) {
        $startpos = stripos( $query, "?" );
        $base = substr( $query, 0, $startpos + 1 );

        $query_str = substr( $query, $startpos + 1 );
        $query_str = explode( "&", $query_str );

        foreach ( $query_str as &$value ) {
            $tmp = explode( "=", $value );

            if ( isset( $tmp[1] ) ) {
                $tmp[1] = urlencode( urldecode( $tmp[1] ) );
            }
            $value = implode( "=", $tmp );
        }

        $encoded_url = implode( "&", $query_str );
        $encoded_url = $base . $encoded_url;

        return $encoded_url;
    }

    /**
     * Function to get the health status of the bound OpenContent
     * @return mixed
     */
    public function get_oc_health() {
        $ch = curl_init();
        curl_setopt( $ch, CURLOPT_URL, $this->oc_base_url . 'health' );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
        curl_setopt( $ch, CURLOPT_TIMEOUT, 5 );
        curl_exec( $ch );
        $response = curl_getinfo( $ch );
        curl_close( $ch );

        return $response;
    }
}